@@ -1,4 +1,4 @@
-import { GlobalOverrider } from "newtab/test/unit/utils";
+import { GlobalOverrider } from "asrouter/tests/unit/utils";
 import {
   MultiStageAboutWelcome,
   SecondaryCTA,
@@ -6,12 +6,19 @@ import {
   ProgressBar,
   WelcomeScreen,
 } from "content-src/components/MultiStageAboutWelcome";
-import { Themes } from "content-src/components/Themes";
+import { SingleSelect } from "content-src/components/SingleSelect";
 import React from "react";
 import { shallow, mount } from "enzyme";
 import { AboutWelcomeDefaults } from "modules/AboutWelcomeDefaults.sys.mjs";
 import { AboutWelcomeUtils } from "content-src/lib/aboutwelcome-utils.mjs";
 
+const spinEventLoop = async () => {
+  // Spin the event loop to allow the useEffect hooks to execute,
+  // any promises to resolve, and re-rendering to happen after the
+  // promises have updated the state/props
+  await new Promise(resolve => setTimeout(resolve, 0));
+};
+
 describe("MultiStageAboutWelcome module", () => {
   let globals;
   let sandbox;
@@ -27,8 +34,12 @@ describe("MultiStageAboutWelcome module", () => {
   beforeEach(async () => {
     globals = new GlobalOverrider();
     globals.set({
+      AWEvaluateScreenTargeting: () => {},
       AWGetSelectedTheme: () => Promise.resolve("automatic"),
+      AWGetInstalledAddons: () => Promise.resolve(["test-addon-id"]),
+      AWGetUnhandledCampaignAction: () => Promise.resolve(false),
       AWSendEventTelemetry: () => {},
+      AWSendToParent: () => {},
       AWWaitForMigrationClose: () => Promise.resolve(),
       AWSelectTheme: () => Promise.resolve(),
       AWFinish: () => Promise.resolve(),
@@ -50,10 +61,7 @@ describe("MultiStageAboutWelcome module", () => {
 
     it("should pass activeTheme and initialTheme props to WelcomeScreen", async () => {
       let wrapper = mount(<MultiStageAboutWelcome {...DEFAULT_PROPS} />);
-      // Spin the event loop to allow the useEffect hooks to execute,
-      // any promises to resolve, and re-rendering to happen after the
-      // promises have updated the state/props
-      await new Promise(resolve => setTimeout(resolve, 0));
+      await spinEventLoop();
       // sync up enzyme's representation with the real DOM
       wrapper.update();
 
@@ -65,6 +73,19 @@ describe("MultiStageAboutWelcome module", () => {
       );
     });
 
+    it("should fetch a list of installed Addons", async () => {
+      let wrapper = mount(<MultiStageAboutWelcome {...DEFAULT_PROPS} />);
+      await spinEventLoop();
+      // sync up enzyme's representation with the real DOM
+      wrapper.update();
+
+      let welcomeScreenWrapper = wrapper.find(WelcomeScreen);
+      assert.strictEqual(
+        welcomeScreenWrapper.prop("installedAddons")[0],
+        "test-addon-id"
+      );
+    });
+
     it("should handle primary Action", () => {
       const screens = [
         {
@@ -200,21 +221,22 @@ describe("MultiStageAboutWelcome module", () => {
 
   describe("WelcomeScreen component", () => {
     describe("easy setup screen", () => {
-      const screen = AboutWelcomeDefaults.getDefaults().screens.find(
+      const easySetupScreen = AboutWelcomeDefaults.getDefaults().screens.find(
         s => s.id === "AW_EASY_SETUP_NEEDS_DEFAULT_AND_PIN"
       );
       let EASY_SETUP_SCREEN_PROPS;
 
       beforeEach(() => {
         EASY_SETUP_SCREEN_PROPS = {
-          id: screen.id,
-          content: screen.content,
-          messageId: `${DEFAULT_PROPS.message_id}_${screen.id}`,
+          id: easySetupScreen.id,
+          content: easySetupScreen.content,
+          messageId: `${DEFAULT_PROPS.message_id}_${easySetupScreen.id}`,
           UTMTerm: DEFAULT_PROPS.utm_term,
           flowParams: null,
           totalNumberOfScreens: 1,
           setScreenMultiSelects: sandbox.stub(),
           setActiveMultiSelect: sandbox.stub(),
+          setActiveSingleSelectSelection: sandbox.stub(),
         };
       });
 
@@ -350,7 +372,7 @@ describe("MultiStageAboutWelcome module", () => {
       });
 
       it("should check this.props.activeTheme in the rendered input", () => {
-        const wrapper = shallow(<Themes {...THEME_SCREEN_PROPS} />);
+        const wrapper = shallow(<SingleSelect {...THEME_SCREEN_PROPS} />);
 
         const selectedThemeInput = wrapper.find(".theme input[checked=true]");
         assert.strictEqual(
@@ -384,68 +406,200 @@ describe("MultiStageAboutWelcome module", () => {
       });
     });
 
-    describe("Wallpaper screen", () => {
-      let WALLPAPER_SCREEN_PROPS;
+    describe("Single select picker screen", () => {
+      let SINGLE_SELECT_SCREEN_PROPS;
       beforeEach(() => {
-        WALLPAPER_SCREEN_PROPS = {
+        SINGLE_SELECT_SCREEN_PROPS = {
           content: {
-            title: "test title",
-            subtitle: "test subtitle",
+            title: {
+              raw: "Test title",
+            },
+            subtitle: {
+              raw: "Test subtitle",
+            },
             tiles: {
-              type: "theme",
-              category: {
-                type: "wallpaper",
-                action: {
-                  type: "MULTI_ACTION",
-                  data: {
-                    actions: [
-                      {
-                        type: "SET_PREF",
-                        data: {
-                          pref: {
-                            name: "test-dark",
-                          },
-                        },
-                      },
-                      {
-                        type: "SET_PREF",
-                        data: {
-                          pref: {
-                            name: "test-light",
-                          },
-                        },
-                      },
-                    ],
-                  },
-                },
-              },
+              type: "single-select",
+              selected: "test1",
               action: {
-                theme: "<event>",
+                picker: "<event>",
               },
               data: [
                 {
-                  theme: "mountain",
-                  type: "light",
+                  id: "test1",
+                  label: {
+                    raw: "test1 label",
+                  },
+                  action: {
+                    type: "SET_PREF",
+                    data: {
+                      pref: {
+                        name: "sidebar.revamp",
+                        value: true,
+                      },
+                    },
+                  },
+                },
+                {
+                  defaultValue: true,
+                  id: "test2",
+                  label: {
+                    raw: "test2 label",
+                  },
+                  flair: {
+                    text: {
+                      raw: "New!",
+                    },
+                  },
+                  action: {
+                    type: "SET_PREF",
+                    data: {
+                      pref: {
+                        name: "sidebar.revamp",
+                        value: false,
+                      },
+                    },
+                  },
                 },
               ],
             },
-            primary_button: {
-              action: {},
-              label: "test button",
+            secondary_button: {
+              label: {
+                raw: "Skip this step",
+              },
+              action: {
+                navigate: true,
+              },
+              has_arrow_icon: true,
             },
           },
           navigate: sandbox.stub(),
-          setActiveTheme: sandbox.stub(),
+          setActiveSingleSelectSelection: sandbox.stub(),
+          setActiveMultiSelect: sandbox.stub(),
+          setScreenMultiSelects: sandbox.stub(),
         };
         sandbox.stub(AboutWelcomeUtils, "handleUserAction").resolves();
       });
-      it("should handle wallpaper click", () => {
-        const wrapper = mount(<WelcomeScreen {...WALLPAPER_SCREEN_PROPS} />);
-        const wallpaperOptions = wrapper.find(
-          ".tiles-theme-section .theme input[name='mountain']"
+      it("should select the configured default value if present", async () => {
+        const wrapper = mount(
+          <WelcomeScreen {...SINGLE_SELECT_SCREEN_PROPS} />
         );
-        wallpaperOptions.simulate("click");
-        assert.calledTwice(AboutWelcomeUtils.handleUserAction);
+        assert.ok(
+          wrapper
+            .find(".tiles-single-select-section .select-item .test1")
+            .exists()
+        );
+      });
+      it("should preselect the active value if present", async () => {
+        SINGLE_SELECT_SCREEN_PROPS.activeSingleSelectSelections = {
+          "tile-0": ["test2"],
+        };
+        const wrapper = mount(
+          <WelcomeScreen {...SINGLE_SELECT_SCREEN_PROPS} />
+        );
+        assert.ok(
+          wrapper
+            .find(".tiles-single-select-section .select-item .test2")
+            .exists()
+        );
+      });
+      it("should handle item click", () => {
+        const wrapper = mount(
+          <WelcomeScreen {...SINGLE_SELECT_SCREEN_PROPS} />
+        );
+        const selectOption = wrapper.find(
+          ".tiles-single-select-section .select-item input[value='test1']"
+        );
+        selectOption.simulate("click");
+        assert.calledOnce(AboutWelcomeUtils.handleUserAction);
+      });
+      it("should handle item key down selection", () => {
+        const wrapper = mount(
+          <WelcomeScreen {...SINGLE_SELECT_SCREEN_PROPS} />
+        );
+        const selectOption = wrapper.find(
+          ".tiles-single-select-section .select-item input[value='test1']"
+        );
+        selectOption.simulate("keydown", { key: "Enter" });
+        assert.calledOnce(AboutWelcomeUtils.handleUserAction);
+      });
+      it("should render flair", () => {
+        const wrapper = mount(
+          <WelcomeScreen {...SINGLE_SELECT_SCREEN_PROPS} />
+        );
+        const flair = wrapper.find(".flair");
+        assert.ok(flair.exists());
+        assert.ok(flair.text() === "New!");
+      });
+      it("should automatically trigger the selected tile's action for an approved action", () => {
+        SINGLE_SELECT_SCREEN_PROPS.content.tiles.autoTrigger = true;
+        mount(<WelcomeScreen {...SINGLE_SELECT_SCREEN_PROPS} />);
+        assert.calledOnce(AboutWelcomeUtils.handleUserAction);
+      });
+      it("should not trigger the selected tile's action for an unapproved action", () => {
+        SINGLE_SELECT_SCREEN_PROPS.content.tiles.autoTrigger = true;
+        SINGLE_SELECT_SCREEN_PROPS.content.tiles.data[0].action = "OPEN_URL";
+
+        mount(<WelcomeScreen {...SINGLE_SELECT_SCREEN_PROPS} />);
+
+        assert.notCalled(AboutWelcomeUtils.handleUserAction);
+      });
+      it("should not trigger the selected tile's action for an unapproved pref with SET_PREF action", () => {
+        SINGLE_SELECT_SCREEN_PROPS.content.tiles.autoTrigger = true;
+        SINGLE_SELECT_SCREEN_PROPS.content.tiles.data[0].action.data.pref =
+          "unapproved.pref";
+
+        mount(<WelcomeScreen {...SINGLE_SELECT_SCREEN_PROPS} />);
+
+        assert.notCalled(AboutWelcomeUtils.handleUserAction);
+      });
+      it("should trigger all of the selected tile's actions if MULTI_ACTION is used with SET_PREF and allowed prefs", () => {
+        SINGLE_SELECT_SCREEN_PROPS.content.tiles.autoTrigger = true;
+        SINGLE_SELECT_SCREEN_PROPS.content.tiles.data[0].action = {
+          type: "MULTI_ACTION",
+          data: {
+            actions: [
+              {
+                type: "SET_PREF",
+                data: {
+                  pref: {
+                    name: "sidebar.revamp",
+                  },
+                },
+              },
+              {
+                type: "SET_PREF",
+                data: {
+                  pref: {
+                    name: "sidebar.verticalTabs",
+                  },
+                },
+              },
+            ],
+          },
+        };
+        mount(<WelcomeScreen {...SINGLE_SELECT_SCREEN_PROPS} />);
+        assert.calledOnce(AboutWelcomeUtils.handleUserAction);
+      });
+      it("should not trigger any of the selected tile's action if MULTI_ACTION is used with one unallowed pref", () => {
+        SINGLE_SELECT_SCREEN_PROPS.content.tiles.autoTrigger = true;
+        SINGLE_SELECT_SCREEN_PROPS.content.tiles.data[0].action = {
+          type: "MULTI_ACTION",
+          data: {
+            actions: [
+              {
+                type: "OPEN_URL",
+              },
+              {
+                type: "SET_PREF",
+                data: {
+                  pref: "sidebar.verticalTabs",
+                },
+              },
+            ],
+          },
+        };
+        mount(<WelcomeScreen {...SINGLE_SELECT_SCREEN_PROPS} />);
+        assert.notCalled(AboutWelcomeUtils.handleUserAction);
       });
     });
 
@@ -491,10 +645,14 @@ describe("MultiStageAboutWelcome module", () => {
         };
         const finishStub = sandbox.stub(global, "AWFinish");
         const wrapper = mount(<WelcomeScreen {...SCREEN_PROPS} />);
-
+        let telemetrySpy = sandbox.spy(
+          AboutWelcomeUtils,
+          "sendDismissTelemetry"
+        );
         wrapper.find(".dismiss-button").simulate("click");
 
         assert.calledOnce(finishStub);
+        assert.calledOnce(telemetrySpy);
       });
       it("should handle SHOW_ICECAT_ACCOUNTS", () => {
         TEST_ACTION.type = "SHOW_ICECAT_ACCOUNTS";
@@ -811,7 +969,7 @@ describe("MultiStageAboutWelcome module", () => {
           const wrapper = mount(
             <WelcomeScreen
               {...PREF_SCREEN_PROPS}
-              activeMultiSelect={["checkbox-1"]}
+              activeMultiSelect={{ "tile-0": ["checkbox-1"] }}
             />
           );
           wrapper.find(".primary").simulate("click");
@@ -844,7 +1002,7 @@ describe("MultiStageAboutWelcome module", () => {
           const wrapper = mount(
             <WelcomeScreen
               {...PREF_SCREEN_PROPS}
-              activeMultiSelect={["checkbox-2"]}
+              activeMultiSelect={{ "tile-0": ["checkbox-2"] }}
             />
           );
           wrapper.find(".primary").simulate("click");
@@ -883,7 +1041,7 @@ describe("MultiStageAboutWelcome module", () => {
           const wrapper = mount(
             <WelcomeScreen
               {...PREF_SCREEN_PROPS}
-              activeMultiSelect={["checkbox-1", "checkbox-2"]}
+              activeMultiSelect={{ "tile-0": ["checkbox-1", "checkbox-2"] }}
             />
           );
           wrapper.find(".primary").simulate("click");
@@ -920,6 +1078,263 @@ describe("MultiStageAboutWelcome module", () => {
           AboutWelcomeUtils.handleUserAction.resetHistory();
         }
       });
+      it("Should handle a campaign action when applicable", async () => {
+        let actionSpy = sandbox.spy(AboutWelcomeUtils, "handleCampaignAction");
+        let telemetrySpy = sandbox.spy(
+          AboutWelcomeUtils,
+          "sendActionTelemetry"
+        );
+
+        globals.set("AWGetUnhandledCampaignAction", () =>
+          Promise.resolve("SET_DEFAULT_BROWSER")
+        );
+        // Return true when "HANDLE_CAMPAIGN_ACTION" is sent to parent
+        globals.set("AWSendToParent", () => Promise.resolve(true));
+        const screens = [
+          {
+            content: {
+              title: "test title",
+            },
+          },
+        ];
+        const TEST_PROPS = {
+          defaultScreens: screens,
+          message_id: "DEFAULT_ABOUTWELCOME",
+          startScreen: 0,
+        };
+        let wrapper = mount(<MultiStageAboutWelcome {...TEST_PROPS} />);
+        await spinEventLoop();
+        wrapper.update();
+        assert.calledOnce(actionSpy);
+        // If campaign is handled, we should send telemetry
+        assert.calledOnce(telemetrySpy);
+        assert.equal(telemetrySpy.firstCall.args[1], "CAMPAIGN_ACTION");
+        globals.restore();
+      });
+      it("Should not handle a campaign action when the action has already been handled", async () => {
+        let actionSpy = sandbox.spy(AboutWelcomeUtils, "handleCampaignAction");
+        let telemetrySpy = sandbox.spy(
+          AboutWelcomeUtils,
+          "sendActionTelemetry"
+        );
+
+        globals.set("AWGetUnhandledCampaignAction", () =>
+          Promise.resolve(false)
+        );
+        const screens = [
+          {
+            content: {
+              title: "test title",
+            },
+          },
+        ];
+        const TEST_PROPS = {
+          defaultScreens: screens,
+          message_id: "DEFAULT_ABOUTWELCOME",
+          startScreen: 0,
+        };
+        let wrapper = mount(<MultiStageAboutWelcome {...TEST_PROPS} />);
+        await spinEventLoop();
+        wrapper.update();
+        assert.notCalled(actionSpy);
+        assert.notCalled(telemetrySpy);
+        globals.restore();
+      });
+      it("Should not send telemetrty when campaign action handling fails", async () => {
+        let actionSpy = sandbox.spy(AboutWelcomeUtils, "handleCampaignAction");
+        let telemetrySpy = sandbox.spy(
+          AboutWelcomeUtils,
+          "sendActionTelemetry"
+        );
+
+        globals.set("AWGetUnhandledCampaignAction", () =>
+          Promise.resolve("SET_DEFAULT_BROWSER")
+        );
+
+        // Return undefined when "HANDLE_CAMPAIGN_ACTION" is sent to parent as
+        // though "AWPage:HANDLE_CAMPAIGN_ACTION" case did not return true due
+        // to failure executing action or the campaign handled pref being true
+        globals.set("AWSendToParent", () => Promise.resolve(undefined));
+        const screens = [
+          {
+            content: {
+              title: "test title",
+            },
+          },
+        ];
+        const TEST_PROPS = {
+          defaultScreens: screens,
+          message_id: "DEFAULT_ABOUTWELCOME",
+          startScreen: 0,
+        };
+        let wrapper = mount(<MultiStageAboutWelcome {...TEST_PROPS} />);
+        await spinEventLoop();
+        wrapper.update();
+        assert.calledOnce(actionSpy);
+        // If campaign handling fails, we should not send telemetry
+        assert.notCalled(telemetrySpy);
+        globals.restore();
+      });
+      it("should handle action from matched tile in tiles array", () => {
+        const TILE_ID = "test-tile-id";
+        SCREEN_PROPS.content.tiles = [
+          {
+            data: [
+              {
+                id: TILE_ID,
+                action: {
+                  type: "OPEN_URL",
+                  data: { args: "https://example.com" },
+                },
+              },
+            ],
+          },
+        ];
+        const wrapper = mount(<WelcomeScreen {...SCREEN_PROPS} />);
+
+        wrapper.instance().handleAction({
+          currentTarget: { value: TILE_ID },
+          source: "test-source",
+          name: "test-click",
+        });
+
+        const [calledArg] = AboutWelcomeUtils.handleUserAction.firstCall.args;
+        const url = new URL(calledArg.data.args);
+
+        assert.equal(calledArg.type, "OPEN_URL");
+        assert.equal(url.hostname, "example.com");
+      });
+    });
+
+    describe("#handleUserAction", () => {
+      let SCREEN_PROPS;
+      let TEST_ACTION;
+      let awSendToParentStub;
+      let finishStub;
+      let handleUserActionSpy;
+      beforeEach(() => {
+        SCREEN_PROPS = {
+          content: {
+            primary_button: {
+              action: { type: "TEST_ACTION" },
+              label: "test button",
+            },
+          },
+        };
+        awSendToParentStub = sandbox.stub();
+        globals.set("AWSendToParent", awSendToParentStub);
+        finishStub = sandbox.stub(global, "AWFinish");
+        handleUserActionSpy = sandbox.spy(
+          AboutWelcomeUtils,
+          "handleUserAction"
+        );
+
+        TEST_ACTION = SCREEN_PROPS.content.primary_button.action;
+      });
+
+      afterEach(() => {
+        handleUserActionSpy.restore();
+        finishStub.restore();
+        globals.restore();
+      });
+
+      it("Should dismiss when resolve boolean is true and needAwait true", async () => {
+        TEST_ACTION.dismiss = "actionResult";
+        TEST_ACTION.needsAwait = true;
+        // `needsAwait` is true, so the handleUserAction function should return a `Promise<boolean>`
+        awSendToParentStub.callsFake(
+          () => Promise.resolve(true) // Resolves to a boolean for awaited calls
+        );
+
+        const wrapper = mount(<WelcomeScreen {...SCREEN_PROPS} />);
+        wrapper.find(".primary").simulate("click");
+
+        // Assert click of primary button calls handleUserAction
+        assert.calledOnce(handleUserActionSpy);
+        assert.equal(handleUserActionSpy.firstCall.args[0].type, "TEST_ACTION");
+        // handleUserAction returns a Promise, so let's let the microtask queue
+        // flush so that anything waiting for the handleUserAction Promise to
+        // resolve can run.
+        await new Promise(resolve => queueMicrotask(resolve));
+
+        // Check handleUserAction return value is a `Promise<boolean>`
+        assert.ok(handleUserActionSpy.firstCall.returnValue instanceof Promise);
+        const awaitedResult = await handleUserActionSpy.firstCall.returnValue;
+        // Check that the result is a boolean
+        assert.equal(
+          typeof awaitedResult,
+          "boolean",
+          "The awaited call should return a boolean."
+        );
+        assert.equal(
+          awaitedResult,
+          true,
+          "The awaited call should resolve to true, as per the mock return value."
+        );
+        // Check AWFinish gets called when awaited actionResult value is true
+        assert.calledOnce(finishStub);
+      });
+
+      it("Should not dismiss when resolve boolean is false and needAwait true", async () => {
+        TEST_ACTION.dismiss = "actionResult";
+        TEST_ACTION.needsAwait = true;
+        // `needsAwait` is true, so the handleUserAction function should return a `Promise<boolean>`
+        awSendToParentStub.callsFake(
+          () => Promise.resolve(false) // Resolves to a boolean for awaited calls
+        );
+
+        const wrapper = mount(<WelcomeScreen {...SCREEN_PROPS} />);
+        wrapper.find(".primary").simulate("click");
+
+        // Assert click of primary button calls handleUserAction
+        assert.calledOnce(handleUserActionSpy);
+        assert.equal(handleUserActionSpy.firstCall.args[0].type, "TEST_ACTION");
+        await new Promise(resolve => queueMicrotask(resolve));
+
+        // Check handleUserAction return value is a `Promise<boolean>`
+        assert.ok(handleUserActionSpy.firstCall.returnValue instanceof Promise);
+        const awaitedResult = await handleUserActionSpy.firstCall.returnValue;
+        // Check that the result is a boolean
+        assert.equal(
+          typeof awaitedResult,
+          "boolean",
+          "The awaited call should return a boolean."
+        );
+        assert.equal(
+          awaitedResult,
+          false,
+          "The awaited call should resolve to false, as per the mock return value."
+        );
+        // Check AWFinish not get called when awaited actionResult value is false
+        assert.notCalled(finishStub);
+      });
+
+      it("Should dismiss when true and handleUserAction not awaited", async () => {
+        TEST_ACTION.dismiss = true;
+        // `needsAwait` is not set, so the handleUserAction function should return a `Promise<undefined>`
+        awSendToParentStub.callsFake(
+          () => Promise.resolve(undefined) // Resolves to a undefined for non awaited calls
+        );
+
+        const wrapper = mount(<WelcomeScreen {...SCREEN_PROPS} />);
+        wrapper.find(".primary").simulate("click");
+
+        // Assert click of primary button calls handleUserAction
+        assert.calledOnce(handleUserActionSpy);
+        assert.equal(handleUserActionSpy.firstCall.args[0].type, "TEST_ACTION");
+        await new Promise(resolve => queueMicrotask(resolve));
+
+        // Check handleUserAction return value is a `Promise<undefined>`
+        assert.ok(handleUserActionSpy.firstCall.returnValue instanceof Promise);
+        const awaitedResult = await handleUserActionSpy.firstCall.returnValue;
+        assert.equal(
+          awaitedResult,
+          undefined,
+          "The awaited call should resolve to undefined, as per the mock return value."
+        );
+        // Check AWFinish gets called when dismiss is true for non awaited calls
+        assert.calledOnce(finishStub);
+      });
     });
   });
 });
