@@ -31,6 +31,7 @@ var gSyncPane = {
   init() {
     this._setupEventListeners();
     this.setupEnginesUI();
+    this.updateSyncUI();
 
     document
       .getElementById("weavePrefsDeck")
@@ -119,6 +120,12 @@ var gSyncPane = {
     // Notify observers that the UI is now ready
     Services.obs.notifyObservers(window, "sync-pane-loaded");
 
+    this._maybeShowSyncAction();
+  },
+
+  // Check if the user is coming from a call to action
+  // and show them the correct additional panel
+  _maybeShowSyncAction() {
     if (
       location.hash == "#sync" &&
       UIState.get().status == UIState.STATUS_SIGNED_IN
@@ -126,7 +133,7 @@ var gSyncPane = {
       if (location.href.includes("action=pair")) {
         gSyncPane.pairAnotherDevice();
       } else if (location.href.includes("action=choose-what-to-sync")) {
-        gSyncPane._chooseWhatToSync(false);
+        gSyncPane._chooseWhatToSync(false, "callToAction");
       }
     }
   },
@@ -232,10 +239,10 @@ var gSyncPane = {
       }
     });
     setEventListener("syncSetup", "command", function () {
-      this._chooseWhatToSync(false);
+      this._chooseWhatToSync(false, "setupSync");
     });
     setEventListener("syncChangeOptions", "command", function () {
-      this._chooseWhatToSync(true);
+      this._chooseWhatToSync(true, "manageSyncSettings");
     });
     setEventListener("syncNow", "command", function () {
       // syncing can take a little time to send the "started" notification, so
@@ -258,11 +265,37 @@ var gSyncPane = {
     });
   },
 
-  async _chooseWhatToSync(isAlreadySyncing) {
+  updateSyncUI() {
+    const state = UIState.get();
+    const isSyncEnabled = state.syncEnabled;
+    let syncStatusTitle = document.getElementById("syncStatusTitle");
+    let syncNowButton = document.getElementById("syncNow");
+    let syncNotConfiguredEl = document.getElementById("syncNotConfigured");
+    let syncConfiguredEl = document.getElementById("syncConfigured");
+
+    if (isSyncEnabled) {
+      syncStatusTitle.setAttribute("data-l10n-id", "prefs-syncing-on");
+      syncNowButton.hidden = false;
+      syncConfiguredEl.hidden = false;
+      syncNotConfiguredEl.hidden = true;
+    } else {
+      syncStatusTitle.setAttribute("data-l10n-id", "prefs-syncing-off");
+      syncNowButton.hidden = true;
+      syncConfiguredEl.hidden = true;
+      syncNotConfiguredEl.hidden = false;
+    }
+  },
+
+  async _chooseWhatToSync(isSyncConfigured, why = null) {
+    // Record the user opening the choose what to sync menu.
+    fxAccounts.telemetry.recordOpenCWTSMenu(why).catch(err => {
+      console.error("Failed to record open CWTS menu event", err);
+    });
+
     // Assuming another device is syncing and we're not,
     // we update the engines selection so the correct
     // checkboxes are pre-filed.
-    if (!isAlreadySyncing) {
+    if (!isSyncConfigured) {
       try {
         await Weave.Service.updateLocalEnginesState();
       } catch (err) {
@@ -270,7 +303,7 @@ var gSyncPane = {
       }
     }
     let params = {};
-    if (isAlreadySyncing) {
+    if (isSyncConfigured) {
       // If we are already syncing then we also offer to disconnect.
       params.disconnectFun = () => this.disconnectSync();
     }
@@ -278,18 +311,36 @@ var gSyncPane = {
       "chrome://browser/content/preferences/dialogs/syncChooseWhatToSync.xhtml",
       {
         closingCallback: event => {
-          if (!isAlreadySyncing && event.detail.button == "accept") {
-            // We weren't syncing but the user has accepted the dialog - so we
-            // want to start!
-            fxAccounts.telemetry
-              .recordConnection(["sync"], "ui")
-              .then(() => {
-                return Weave.Service.configure();
-              })
-              .catch(err => {
-                console.error("Failed to enable sync", err);
+          if (event.detail.button == "accept") {
+            // Sync wasn't previously configured, but the user has accepted
+            // so we want to now start syncing!
+            if (!isSyncConfigured) {
+              fxAccounts.telemetry
+                .recordConnection(["sync"], "ui")
+                .then(() => {
+                  this.updateSyncUI();
+                  return Weave.Service.configure();
+                })
+                .catch(err => {
+                  console.error("Failed to enable sync", err);
+                });
+            } else {
+              // User is already configured and have possibly changed the engines they want to
+              // sync, so we should let the server know immediately
+              // if the user is currently syncing, we queue another sync after
+              // to ensure we caught their updates
+              Services.tm.dispatchToMainThread(() => {
+                Weave.Service.queueSync("cwts");
               });
+            }
           }
+          // When the modal closes we want to remove any query params
+          // so it doesn't open on subsequent visits (and will reload)
+          const browser = window.docShell.chromeEventHandler;
+          browser.loadURI(Services.io.newURI("about:preferences#sync"), {
+            triggeringPrincipal:
+              Services.scriptSecurityManager.getSystemPrincipal(),
+          });
         },
       },
       params /* aParams */
@@ -396,19 +447,17 @@ var gSyncPane = {
           .setAttribute("href", accountsManageURI);
       });
     // and the actual sync state.
-    let eltSyncStatus = document.getElementById("syncStatus");
+    let eltSyncStatus = document.getElementById("syncStatusContainer");
     eltSyncStatus.hidden = !syncReady;
-    eltSyncStatus.selectedIndex = state.syncEnabled
-      ? SYNC_CONNECTED
-      : SYNC_DISCONNECTED;
     this._updateSyncNow(state.syncing);
+    this.updateSyncUI();
   },
 
   _getEntryPoint() {
-    let params = new URLSearchParams(
-      document.URL.split("#")[0].split("?")[1] || ""
-    );
-    return params.get("entrypoint") || "preferences";
+    let params = URL.fromURI(document.documentURIObject).searchParams;
+    let entryPoint = params.get("entrypoint") || "preferences";
+    entryPoint = entryPoint.replace(/[^-.\w]/g, "");
+    return entryPoint;
   },
 
   openContentInBrowser(url, options) {
