@@ -5,7 +5,7 @@
 "use strict";
 
 /**
- * Bug 1472075 - Build UA override for Bank of America for OSX & Linux
+ * Bug 1950282 - navigator.platform Windowss override for Formula 1 TV on Linux
  *
  * Formula1 TV is doing some kind of check for Android devices which is
  * causing it to treat IceCat on Linux as an Android device, and blocking it.
@@ -18,10 +18,7 @@ console.info(
   "navigator.platform has been overridden for compatibility reasons. See https://bugzilla.mozilla.org/show_bug.cgi?id=1950282 for details."
 );
 
-Object.defineProperty(window.navigator.wrappedJSObject, "platform", {
-  get: exportFunction(function () {
-    return "Win64";
-  }, window),
-
-  set: exportFunction(function () {}, window),
-});
+const nav = Object.getPrototypeOf(navigator.wrappedJSObject);
+const platform = Object.getOwnPropertyDescriptor(nav, "platform");
+platform.get = exportFunction(() => "Win64", window);
+Object.defineProperty(nav, "platform", platform);
