@@ -97,13 +97,21 @@ async function openAboutDevtoolsToolbox(
   ok(inspectButton, `Inspect button for ${targetText} appeared`);
   inspectButton.click();
   const onToolboxReady = gDevTools.once("toolbox-ready");
-  await Promise.all([
-    waitForAboutDebuggingRequests(win.AboutDebugging.store),
-    shouldWaitToolboxReady ? onToolboxReady : Promise.resolve(),
-  ]);
 
-  // WebExtension open a toolbox in a dedicated window
-  if (isWebExtension) {
+  info("Wait for about debugging requests to settle");
+  await waitForAboutDebuggingRequests(win.AboutDebugging.store);
+
+  if (shouldWaitToolboxReady) {
+    info("Wait for onToolboxReady");
+    await onToolboxReady;
+  }
+
+  const { runtimes } = win.AboutDebugging.store.getState();
+  const isOnThisIceCat = runtimes.selectedRuntimeId === "this-icecat";
+  const isLocalWebExtension = isWebExtension && isOnThisIceCat;
+
+  // Local WebExtension toolboxes open in a dedicated window
+  if (isLocalWebExtension) {
     const toolbox = await onToolboxReady;
     // For some reason the test helpers prevents the toolbox from being automatically focused on opening,
     // whereas it is IRL.
@@ -128,12 +136,14 @@ async function openAboutDevtoolsToolbox(
     };
   }
 
+  info("Wait until a new tab is opened");
   await waitUntil(() => tab.nextElementSibling);
 
   info("Wait for about:devtools-toolbox tab will be selected");
   const devtoolsTab = tab.nextElementSibling;
   await waitUntil(() => gBrowser.selectedTab === devtoolsTab);
   const devtoolsBrowser = gBrowser.selectedBrowser;
+  info("Wait for about:devtools-toolbox tab to have the expected URL");
   await waitUntil(() =>
     devtoolsBrowser.contentWindow.location.href.startsWith(
       "about:devtools-toolbox?"
@@ -164,7 +174,11 @@ async function closeAboutDevtoolsToolbox(
   const devtoolsBrowser = devtoolsTab.linkedBrowser;
   const devtoolsWindow = devtoolsBrowser.contentWindow;
   const toolbox = getToolbox(devtoolsWindow);
-  await toolbox.commands.client.waitForRequestsToSettle();
+
+  info("Wait for requests to settle");
+  await toolbox.commands.client.waitForRequestsToSettle({
+    ignoreOrphanedFronts: true,
+  });
 
   info("Close about:devtools-toolbox page");
   const onToolboxDestroyed = gDevTools.once("toolbox-destroyed");
@@ -501,7 +515,9 @@ function clickOnAddonWidget(addonId) {
   ok(browserActionEl, "Got the browserAction button from the browser UI");
 
   info("Show the web extension popup");
-  browserActionEl.firstElementChild.click();
+  browserActionEl
+    .querySelector(".unified-extensions-item-action-button")
+    .click();
 }
 
 // Create basic addon data as the DevToolsClient would return it.
@@ -524,3 +540,41 @@ function createAddonData({
     debuggable: true,
   };
 }
+
+async function connectToLocalIceCat({ runtimeId, runtimeName, deviceName }) {
+  // This is a client to the current IceCat.
+  const clientWrapper = await createLocalClientWrapper();
+
+  // enable USB devices mocks
+  const mocks = new Mocks();
+  const usbClient = mocks.createUSBRuntime(runtimeId, {
+    deviceName,
+    name: runtimeName,
+    clientWrapper,
+  });
+
+  // Wrap a disconnect helper for convenience for the caller.
+  const disconnect = doc =>
+    disconnectFromLocalIceCat({
+      doc,
+      runtimeId,
+      deviceName,
+      mocks,
+    });
+
+  return { disconnect, mocks, usbClient };
+}
+/* exported connectToLocalIceCat */
+
+async function disconnectFromLocalIceCat({
+  doc,
+  mocks,
+  runtimeId,
+  deviceName,
+}) {
+  info("Remove USB runtime");
+  mocks.removeUSBRuntime(runtimeId);
+  mocks.emitUSBUpdate();
+  await waitUntilUsbDeviceIsUnplugged(deviceName, doc);
+}
+/* exported disconnectFromLocalIceCat */
