@@ -8,12 +8,12 @@
 const {
   Component,
   createFactory,
-} = require("resource://devtools/client/shared/vendor/react.js");
+} = require("resource://devtools/client/shared/vendor/react.mjs");
 const {
   connect,
-} = require("resource://devtools/client/shared/redux/visibility-handler-connect.js");
+} = require("resource://devtools/client/shared/vendor/react-redux.js");
 const Actions = require("resource://devtools/client/netmonitor/src/actions/index.js");
-const PropTypes = require("resource://devtools/client/shared/vendor/react-prop-types.js");
+const PropTypes = require("resource://devtools/client/shared/vendor/react-prop-types.mjs");
 const dom = require("resource://devtools/client/shared/vendor/react-dom-factories.js");
 const {
   getFormattedIPAndPort,
@@ -64,16 +64,21 @@ loader.lazyGetter(this, "MDNLink", function () {
   );
 });
 loader.lazyGetter(this, "Rep", function () {
-  return require("resource://devtools/client/shared/components/reps/index.js")
-    .REPS.Rep;
+  return ChromeUtils.importESModule(
+    "resource://devtools/client/shared/components/reps/index.mjs"
+  ).REPS.Rep;
 });
 loader.lazyGetter(this, "MODE", function () {
-  return require("resource://devtools/client/shared/components/reps/index.js")
-    .MODE;
+  return ChromeUtils.importESModule(
+    "resource://devtools/client/shared/components/reps/index.mjs"
+  ).MODE;
 });
 loader.lazyGetter(this, "TreeRow", function () {
   return createFactory(
-    require("resource://devtools/client/shared/components/tree/TreeRow.js")
+    ChromeUtils.importESModule(
+      "resource://devtools/client/shared/components/tree/TreeRow.mjs",
+      { global: "current" }
+    ).default
   );
 });
 loader.lazyRequireGetter(
@@ -104,10 +109,10 @@ const EDIT_AND_RESEND = L10N.getStr("netmonitor.summary.editAndResend");
 const RAW_HEADERS = L10N.getStr("netmonitor.headers.raw");
 const HEADERS_EMPTY_TEXT = L10N.getStr("headersEmptyText");
 const HEADERS_FILTER_TEXT = L10N.getStr("headersFilterText");
-const REQUEST_HEADERS = L10N.getStr("requestHeaders");
-const REQUEST_HEADERS_FROM_UPLOAD = L10N.getStr("requestHeadersFromUpload");
-const RESPONSE_HEADERS = L10N.getStr("responseHeaders");
 const HEADERS_STATUS = L10N.getStr("netmonitor.headers.status");
+const HEADERS_EARLYHINT_STATUS = L10N.getStr(
+  "netmonitor.headers.earlyHintsStatus"
+);
 const HEADERS_VERSION = L10N.getStr("netmonitor.headers.version");
 const HEADERS_TRANSFERRED = L10N.getStr("netmonitor.toolbar.transferred");
 const SUMMARY_STATUS_LEARN_MORE = L10N.getStr("netmonitor.summary.learnMore");
@@ -123,6 +128,116 @@ const HEADERS_ETP = L10N.getStr(
 );
 const HEADERS_PRIORITY = L10N.getStr("netmonitor.headers.requestPriority");
 const HEADERS_DNS = L10N.getStr("netmonitor.headers.dns");
+
+// Order is as displayed
+const HEADERS_CONFIG = {
+  earlyHintsResponseHeaders: {
+    // Key for fetching the data from the backend
+    fetchKey: "earlyHintsResponseHeaders",
+    title: L10N.getStr("earlyHintsResponseHeaders"),
+    // Gets the content to be displayed when switched to the raw headers view
+    rawHeaderValue: ({ headerData }) => {
+      const preHeaderText = headerData.rawHeaders.split("\r\n")[0];
+      return writeHeaderText(headerData.headers, preHeaderText).trim();
+    },
+    // Gets the full display text to be displayed in the header title bar(before the raw toggle button)
+    displayTitle: ({ headerData }) => {
+      const title = HEADERS_CONFIG.earlyHintsResponseHeaders.title;
+      if (headerData.headersSize) {
+        return `${title} (${getFormattedSize(headerData.headersSize, 3)})`;
+      }
+      const rawHeaderValue =
+        HEADERS_CONFIG.earlyHintsResponseHeaders.rawHeaderValue({ headerData });
+      return `${title} (${getFormattedSize(rawHeaderValue.length, 3)})`;
+    },
+  },
+  responseHeaders: {
+    // Key for fetching the data from the backend
+    fetchKey: "responseHeaders",
+    title: L10N.getStr("responseHeaders"),
+    // Gets the content to be displayed when switched to the raw headers view
+    rawHeaderValue: ({ status, statusText, httpVersion, headerData }) => {
+      const preHeaderText = `${httpVersion} ${status} ${statusText}`;
+      return writeHeaderText(headerData.headers, preHeaderText).trim();
+    },
+    // Gets the full display text to be displayed in the header title bar(before the raw toggle button)
+    displayTitle: ({ status, statusText, httpVersion, headerData }) => {
+      const title = HEADERS_CONFIG.responseHeaders.title;
+      if (headerData.headersSize) {
+        return `${title} (${getFormattedSize(headerData.headersSize, 3)})`;
+      }
+      const rawHeaderValue = HEADERS_CONFIG.responseHeaders.rawHeaderValue({
+        httpVersion,
+        status,
+        statusText,
+        headerData,
+      });
+      return `${title} (${getFormattedSize(rawHeaderValue.length, 3)})`;
+    },
+  },
+  requestHeaders: {
+    // See comment above
+    fetchKey: "requestHeaders",
+    title: L10N.getStr("requestHeaders"),
+    // See comment above
+    rawHeaderValue: ({ method, httpVersion, headerData, urlDetails }) => {
+      return getRequestHeadersRawText(
+        method,
+        httpVersion,
+        headerData,
+        urlDetails
+      );
+    },
+    // See comment above
+    displayTitle: ({ method, httpVersion, headerData, urlDetails }) => {
+      const title = HEADERS_CONFIG.requestHeaders.title;
+      if (headerData.headersSize) {
+        return `${title} (${getFormattedSize(headerData.headersSize, 3)})`;
+      }
+      const rawHeaderValue = HEADERS_CONFIG.requestHeaders.rawHeaderValue({
+        method,
+        httpVersion,
+        headerData,
+        urlDetails,
+      });
+      return `${title} (${getFormattedSize(rawHeaderValue.length, 3)})`;
+    },
+  },
+  requestHeadersFromUploadStream: {
+    // See comment above
+    fetchKey: "requestPostData",
+    title: L10N.getStr("requestHeadersFromUpload"),
+    // See comment above
+    rawHeaderValue: ({ headerData, preHeaderText = "" }) => {
+      return writeHeaderText(headerData.headers, preHeaderText).trim();
+    },
+    // See comment above
+    displayTitle: ({ method, httpVersion, headerData, urlDetails }) => {
+      const title = HEADERS_CONFIG.requestHeadersFromUploadStream.title;
+      if (headerData.headersSize) {
+        return `${title} (${getFormattedSize(headerData.headersSize, 3)})`;
+      }
+      let preHeaderText = "";
+      const hostHeader = headerData.headers.find(ele => ele.name === "Host");
+      if (hostHeader) {
+        preHeaderText = `${method} ${
+          urlDetails.url.split(hostHeader.value)[1]
+        } ${httpVersion}`;
+      }
+
+      const rawHeaderValue =
+        HEADERS_CONFIG.requestHeadersFromUploadStream.rawHeaderValue({
+          headerData,
+          preHeaderText,
+        });
+      return `${title} (${getFormattedSize(rawHeaderValue.length, 3)})`;
+    },
+  },
+};
+
+const HEADERS_TO_FETCH = Object.values(HEADERS_CONFIG).map(
+  headers => headers.fetchKey
+);
 /**
  * Headers panel component
  * Lists basic information about the request
@@ -154,18 +269,14 @@ class HeadersPanel extends Component {
     super(props);
 
     this.state = {
-      rawRequestHeadersOpened: false,
-      rawResponseHeadersOpened: false,
-      rawUploadHeadersOpened: false,
+      openedRawHeaders: new Set(),
       lastToggledRawHeader: "",
       filterText: null,
     };
 
     this.getProperties = this.getProperties.bind(this);
     this.getTargetHeaderPath = this.getTargetHeaderPath.bind(this);
-    this.toggleRawResponseHeaders = this.toggleRawResponseHeaders.bind(this);
-    this.toggleRawRequestHeaders = this.toggleRawRequestHeaders.bind(this);
-    this.toggleRawUploadHeaders = this.toggleRawUploadHeaders.bind(this);
+    this.toggleRawHeader = this.toggleRawHeader.bind(this);
     this.renderSummary = this.renderSummary.bind(this);
     this.renderRow = this.renderRow.bind(this);
     this.renderValue = this.renderValue.bind(this);
@@ -176,120 +287,52 @@ class HeadersPanel extends Component {
 
   componentDidMount() {
     const { request, connector } = this.props;
-    fetchNetworkUpdatePacket(connector.requestData, request, [
-      "requestHeaders",
-      "responseHeaders",
-      "requestPostData",
-    ]);
+    fetchNetworkUpdatePacket(connector.requestData, request, HEADERS_TO_FETCH);
   }
 
   // FIXME: https://bugzilla.mozilla.org/show_bug.cgi?id=1774507
   UNSAFE_componentWillReceiveProps(nextProps) {
     const { request, connector } = nextProps;
-    fetchNetworkUpdatePacket(connector.requestData, request, [
-      "requestHeaders",
-      "responseHeaders",
-      "requestPostData",
-    ]);
+    fetchNetworkUpdatePacket(connector.requestData, request, HEADERS_TO_FETCH);
   }
 
-  getHeadersTitle(headers, title) {
-    let result = "";
-    let preHeaderText = "";
-    const {
-      responseHeaders,
-      requestHeaders,
-      httpVersion,
-      status,
-      statusText,
-      method,
-      urlDetails,
-    } = this.props.request;
-    if (headers?.headers.length) {
-      if (!headers.headersSize) {
-        if (title == RESPONSE_HEADERS) {
-          preHeaderText = `${httpVersion} ${status} ${statusText}`;
-          result = `${title} (${getFormattedSize(
-            writeHeaderText(responseHeaders.headers, preHeaderText).length,
-            3
-          )})`;
-        } else {
-          const hostHeader = requestHeaders.headers.find(
-            ele => ele.name === "Host"
-          );
-          if (hostHeader) {
-            preHeaderText = `${method} ${
-              urlDetails.url.split(hostHeader.value)[1]
-            } ${httpVersion}`;
-          }
-          result = `${title} (${getFormattedSize(
-            writeHeaderText(requestHeaders.headers, preHeaderText).length,
-            3
-          )})`;
-        }
-      } else {
-        result = `${title} (${getFormattedSize(headers.headersSize, 3)})`;
-      }
+  // The title to be display in the heading
+  getHeadersDisplayTitle(headerKey) {
+    const headerData = this.props.request[headerKey];
+    if (!headerData?.headers.length) {
+      return "";
     }
 
-    return result;
+    return HEADERS_CONFIG[headerKey].displayTitle({
+      ...this.props.request,
+      headerData,
+    });
   }
 
-  getProperties(headers, title) {
+  getProperties(headerKey) {
     let propertiesResult;
-
-    if (headers?.headers.length) {
-      const headerKey = this.getHeadersTitle(headers, title);
+    const headerData = this.props.request[headerKey];
+    if (headerData?.headers.length) {
       propertiesResult = {
-        [headerKey]: new HeaderList(headers.headers),
+        [headerKey]: this.state.openedRawHeaders.has(headerKey)
+          ? { RAW_HEADERS_ID: headerData.rawHeaders }
+          : new HeaderList(headerData.headers),
       };
-      if (
-        (title === RESPONSE_HEADERS && this.state.rawResponseHeadersOpened) ||
-        (title === REQUEST_HEADERS && this.state.rawRequestHeadersOpened) ||
-        (title === REQUEST_HEADERS_FROM_UPLOAD &&
-          this.state.rawUploadHeadersOpened)
-      ) {
-        propertiesResult = {
-          [headerKey]: { RAW_HEADERS_ID: headers.rawHeaders },
-        };
-      }
     }
     return propertiesResult;
   }
-
-  toggleRawResponseHeaders() {
-    this.setState({
-      rawResponseHeadersOpened: !this.state.rawResponseHeadersOpened,
-      lastToggledRawHeader: "response",
-    });
-  }
-
-  toggleRawRequestHeaders() {
-    this.setState({
-      rawRequestHeadersOpened: !this.state.rawRequestHeadersOpened,
-      lastToggledRawHeader: "request",
-    });
-  }
-
-  toggleRawUploadHeaders() {
-    this.setState({
-      rawUploadHeadersOpened: !this.state.rawUploadHeadersOpened,
-      lastToggledRawHeader: "upload",
-    });
-  }
-
-  /**
-   * Helper method to identify what kind of raw header this is.
-   * Information is in the path variable
-   */
-  getRawHeaderType(path) {
-    if (path.includes(RESPONSE_HEADERS)) {
-      return "RESPONSE";
+  // Toggles the raw headers view on / off
+  toggleRawHeader(headerKey) {
+    const newOpenedRawHeaders = new Set([...this.state.openedRawHeaders]);
+    if (newOpenedRawHeaders.has(headerKey)) {
+      newOpenedRawHeaders.delete(headerKey);
+    } else {
+      newOpenedRawHeaders.add(headerKey);
     }
-    if (path.includes(REQUEST_HEADERS_FROM_UPLOAD)) {
-      return "UPLOAD";
-    }
-    return "REQUEST";
+    this.setState({
+      openedRawHeaders: newOpenedRawHeaders,
+      lastToggledRawHeader: headerKey,
+    });
   }
 
   /**
@@ -315,45 +358,20 @@ class HeadersPanel extends Component {
    * Note that the target header is set by the Search panel.
    */
   getTargetHeaderPath(searchResult) {
-    if (!searchResult) {
+    if (!searchResult || !(searchResult.type in HEADERS_CONFIG)) {
       return null;
     }
-    if (
-      searchResult.type !== "requestHeaders" &&
-      searchResult.type !== "responseHeaders" &&
-      searchResult.type !== "requestHeadersFromUploadStream"
-    ) {
-      return null;
-    }
-    const {
-      request: {
-        requestHeaders,
-        requestHeadersFromUploadStream: uploadHeaders,
-        responseHeaders,
-      },
-    } = this.props;
     // Using `HeaderList` ensures that we'll get the same
     // header index as it's used in the tree.
-    const getPath = (headers, title) => {
-      return (
-        "/" +
-        this.getHeadersTitle(headers, title) +
-        "/" +
-        new HeaderList(headers.headers).headers.findIndex(
-          header => header.name == searchResult.label
-        )
-      );
-    };
-    // Calculate target header path according to the header type.
-    switch (searchResult.type) {
-      case "requestHeaders":
-        return getPath(requestHeaders, REQUEST_HEADERS);
-      case "responseHeaders":
-        return getPath(responseHeaders, RESPONSE_HEADERS);
-      case "requestHeadersFromUploadStream":
-        return getPath(uploadHeaders, REQUEST_HEADERS_FROM_UPLOAD);
-    }
-    return null;
+    const headerData = this.props.request[searchResult.type];
+    return (
+      "/" +
+      searchResult.type +
+      "/" +
+      new HeaderList(headerData.headers).headers.findIndex(
+        header => header.name == searchResult.label
+      )
+    );
   }
 
   /**
@@ -363,43 +381,13 @@ class HeadersPanel extends Component {
   renderRow(props) {
     const { level, path } = props.member;
 
-    const {
-      request: {
-        method,
-        httpVersion,
-        requestHeaders,
-        requestHeadersFromUploadStream: uploadHeaders,
-        responseHeaders,
-        status,
-        statusText,
-        urlDetails,
-      },
-    } = this.props;
-
-    let value;
-    let preHeaderText = "";
     if (path.includes("RAW_HEADERS_ID")) {
-      const rawHeaderType = this.getRawHeaderType(path);
-      switch (rawHeaderType) {
-        case "REQUEST":
-          value = getRequestHeadersRawText(
-            method,
-            httpVersion,
-            requestHeaders,
-            urlDetails
-          );
-          break;
-        case "RESPONSE":
-          preHeaderText = `${httpVersion} ${status} ${statusText}`;
-          value = writeHeaderText(
-            responseHeaders.headers,
-            preHeaderText
-          ).trim();
-          break;
-        case "UPLOAD":
-          value = writeHeaderText(uploadHeaders.headers, preHeaderText).trim();
-          break;
-      }
+      const headerKey = path.split("/")[1];
+
+      const value = HEADERS_CONFIG[headerKey].rawHeaderValue({
+        ...this.props.request,
+        headerData: this.props.request[headerKey],
+      });
 
       let rows;
       if (value) {
@@ -415,9 +403,7 @@ class HeadersPanel extends Component {
           key: path,
           role: "treeitem",
           className: "raw-headers-container",
-          onClick: event => {
-            event.stopPropagation();
-          },
+          onClick: event => event.stopPropagation(),
         },
         td(
           {
@@ -440,26 +426,23 @@ class HeadersPanel extends Component {
     return TreeRow(props);
   }
 
-  renderRawHeadersBtn(key, checked, onChange) {
+  renderRawHeadersBtn(headerKey) {
     return [
       label(
         {
-          key: `${key}RawHeadersBtn`,
+          key: `${headerKey}RawHeadersBtn`,
           className: "raw-headers-toggle",
-          htmlFor: `raw-${key}-checkbox`,
-          onClick: event => {
-            // stop the header click event
-            event.stopPropagation();
-          },
+          onClick: event => event.stopPropagation(),
+          onKeyDown: event => event.stopPropagation(),
         },
         span({ className: "headers-summary-label" }, RAW_HEADERS),
         span(
           { className: "raw-headers-toggle-input" },
           input({
-            id: `raw-${key}-checkbox`,
-            checked,
+            id: `raw-${headerKey}-checkbox`,
+            checked: this.state.openedRawHeaders.has(headerKey),
             className: "devtools-checkbox-toggle",
-            onChange,
+            onChange: () => this.toggleRawHeader(headerKey),
             type: "checkbox",
           })
         )
@@ -492,14 +475,14 @@ class HeadersPanel extends Component {
     );
   }
 
-  getShouldOpen(rawHeader, filterText, targetSearchResult) {
+  getShouldOpen(headerKey, filterText, targetSearchResult) {
     return (item, opened) => {
       // If closed, open panel for these reasons
       //  1.The raw header is switched on or off
       //  2.The filter text is set
       //  3.The search text is set
       if (
-        (!opened && this.state.lastToggledRawHeader === rawHeader) ||
+        (!opened && this.state.lastToggledRawHeader === headerKey) ||
         (!opened && filterText) ||
         (!opened && targetSearchResult)
       ) {
@@ -554,9 +537,6 @@ class HeadersPanel extends Component {
         method,
         remoteAddress,
         remotePort,
-        requestHeaders,
-        requestHeadersFromUploadStream: uploadHeaders,
-        responseHeaders,
         status,
         statusText,
         urlDetails,
@@ -569,126 +549,52 @@ class HeadersPanel extends Component {
         proxyHttpVersion,
         proxyStatus,
         proxyStatusText,
+        earlyHintsStatus,
       },
       openRequestBlockingAndAddUrl,
       openHTTPCustomRequestTab,
       shouldExpandPreview,
       setHeadersUrlPreviewExpanded,
     } = this.props;
-    const {
-      rawResponseHeadersOpened,
-      rawRequestHeadersOpened,
-      rawUploadHeadersOpened,
-      filterText,
-    } = this.state;
 
-    if (
-      (!requestHeaders || !requestHeaders.headers.length) &&
-      (!uploadHeaders || !uploadHeaders.headers.length) &&
-      (!responseHeaders || !responseHeaders.headers.length)
-    ) {
+    const headersDataExists = Object.keys(HEADERS_CONFIG).some(
+      headerKey => this.props.request[headerKey]?.headers.length
+    );
+
+    if (!headersDataExists) {
       return div({ className: "empty-notice" }, HEADERS_EMPTY_TEXT);
     }
 
     const items = [];
 
-    if (responseHeaders?.headers.length) {
-      items.push({
-        component: PropertiesView,
-        componentProps: {
-          object: this.getProperties(responseHeaders, RESPONSE_HEADERS),
-          filterText,
-          targetSearchResult,
-          renderRow: this.renderRow,
-          renderValue: this.renderValue,
-          provider: HeadersProvider,
-          selectPath: this.getTargetHeaderPath,
-          defaultSelectFirstNode: false,
-          enableInput: false,
-          useQuotes: false,
-        },
-        header: this.getHeadersTitle(responseHeaders, RESPONSE_HEADERS),
-        buttons: this.renderRawHeadersBtn(
-          "response",
-          rawResponseHeadersOpened,
-          this.toggleRawResponseHeaders
-        ),
-        id: "responseHeaders",
-        opened: true,
-        shouldOpen: this.getShouldOpen(
-          "response",
-          filterText,
-          targetSearchResult
-        ),
-      });
-    }
-
-    if (requestHeaders?.headers.length) {
-      items.push({
-        component: PropertiesView,
-        componentProps: {
-          object: this.getProperties(requestHeaders, REQUEST_HEADERS),
-          filterText,
-          targetSearchResult,
-          renderRow: this.renderRow,
-          renderValue: this.renderValue,
-          provider: HeadersProvider,
-          selectPath: this.getTargetHeaderPath,
-          defaultSelectFirstNode: false,
-          enableInput: false,
-          useQuotes: false,
-        },
-        header: this.getHeadersTitle(requestHeaders, REQUEST_HEADERS),
-        buttons: this.renderRawHeadersBtn(
-          "request",
-          rawRequestHeadersOpened,
-          this.toggleRawRequestHeaders
-        ),
-        id: "requestHeaders",
-        opened: true,
-        shouldOpen: this.getShouldOpen(
-          "request",
-          filterText,
-          targetSearchResult
-        ),
-      });
-    }
-
-    if (uploadHeaders?.headers.length) {
-      items.push({
-        component: PropertiesView,
-        componentProps: {
-          object: this.getProperties(
-            uploadHeaders,
-            REQUEST_HEADERS_FROM_UPLOAD
+    for (const headerKey in HEADERS_CONFIG) {
+      if (this.props.request[headerKey]?.headers.length) {
+        const { filterText } = this.state;
+        items.push({
+          component: PropertiesView,
+          componentProps: {
+            object: this.getProperties(headerKey),
+            filterText,
+            targetSearchResult,
+            renderRow: this.renderRow,
+            renderValue: this.renderValue,
+            provider: HeadersProvider,
+            selectPath: this.getTargetHeaderPath,
+            defaultSelectFirstNode: false,
+            enableInput: false,
+            useQuotes: false,
+          },
+          header: this.getHeadersDisplayTitle(headerKey),
+          buttons: this.renderRawHeadersBtn(headerKey),
+          id: headerKey,
+          opened: true,
+          shouldOpen: this.getShouldOpen(
+            headerKey,
+            filterText,
+            targetSearchResult
           ),
-          filterText,
-          targetSearchResult,
-          renderRow: this.renderRow,
-          renderValue: this.renderValue,
-          provider: HeadersProvider,
-          selectPath: this.getTargetHeaderPath,
-          defaultSelectFirstNode: false,
-          enableInput: false,
-          useQuotes: false,
-        },
-        header: this.getHeadersTitle(
-          uploadHeaders,
-          REQUEST_HEADERS_FROM_UPLOAD
-        ),
-        buttons: this.renderRawHeadersBtn(
-          "upload",
-          rawUploadHeadersOpened,
-          this.toggleRawUploadHeaders
-        ),
-        id: "uploadHeaders",
-        opened: true,
-        shouldOpen: this.getShouldOpen(
-          "upload",
-          filterText,
-          targetSearchResult
-        ),
-      });
+        });
+      }
     }
 
     const sizeText = L10N.getFormatStrWithNumbers(
@@ -699,6 +605,40 @@ class HeadersPanel extends Component {
 
     const summarySize = this.renderSummary(HEADERS_TRANSFERRED, sizeText);
 
+    let summaryEarlyStatus;
+    if (earlyHintsStatus) {
+      summaryEarlyStatus = div(
+        {
+          key: "headers-summary",
+          className:
+            "tabpanel-summary-container headers-summary headers-earlyhint-status",
+        },
+        span(
+          {
+            className: "tabpanel-summary-label headers-summary-label",
+          },
+          HEADERS_EARLYHINT_STATUS
+        ),
+        span(
+          {
+            className: "tabpanel-summary-value status",
+            "data-code": earlyHintsStatus,
+          },
+          StatusCode({
+            item: {
+              fromCache,
+              fromServiceWorker,
+              status: earlyHintsStatus,
+              statusText: "",
+            },
+          }),
+          MDNLink({
+            url: getHTTPStatusCodeURL(earlyHintsStatus),
+            title: SUMMARY_STATUS_LEARN_MORE,
+          })
+        )
+      );
+    }
     let summaryStatus;
     if (status) {
       summaryStatus = div(
@@ -822,6 +762,7 @@ class HeadersPanel extends Component {
     );
 
     const summaryItems = [
+      summaryEarlyStatus,
       summaryStatus,
       summaryProxyStatus,
       summaryVersion,
