@@ -143,6 +143,7 @@ class IceCatDataProvider {
       responseContent,
       responseCookies,
       responseHeaders,
+      earlyHintsResponseHeaders,
       requestCookies,
       requestHeaders,
       requestPostData,
@@ -154,6 +155,7 @@ class IceCatDataProvider {
       responseContentObj,
       requestHeadersObj,
       responseHeadersObj,
+      earlyHintsResponseHeadersObj,
       postDataObj,
       requestCookiesObj,
       responseCookiesObj,
@@ -162,6 +164,7 @@ class IceCatDataProvider {
       this.fetchResponseContent(responseContent),
       this.fetchRequestHeaders(requestHeaders),
       this.fetchResponseHeaders(responseHeaders),
+      this.fetchEarlyHintResponseHeaders(earlyHintsResponseHeaders),
       this.fetchPostData(requestPostData),
       this.fetchRequestCookies(requestCookies),
       this.fetchResponseCookies(responseCookies),
@@ -174,6 +177,7 @@ class IceCatDataProvider {
       responseContentObj,
       requestHeadersObj,
       responseHeadersObj,
+      earlyHintsResponseHeadersObj,
       postDataObj,
       requestCookiesObj,
       responseCookiesObj,
@@ -220,6 +224,20 @@ class IceCatDataProvider {
     return payload;
   }
 
+  async fetchEarlyHintResponseHeaders(earlyHintsResponseHeaders) {
+    const payload = {};
+    if (earlyHintsResponseHeaders?.headers?.length) {
+      const headers = await fetchHeaders(
+        earlyHintsResponseHeaders,
+        this.getLongString
+      );
+      if (headers) {
+        payload.earlyHintsResponseHeaders = headers;
+      }
+    }
+    return payload;
+  }
+
   async fetchPostData(requestPostData) {
     const payload = {};
     if (requestPostData?.postData) {
@@ -345,9 +363,8 @@ class IceCatDataProvider {
     // In any case, the previous target is destroyed and we can't fetch the stack anymore.
     let stacktrace = [];
     if (!actor.targetFront.isDestroyed()) {
-      const networkContentFront = await actor.targetFront.getFront(
-        "networkContent"
-      );
+      const networkContentFront =
+        await actor.targetFront.getFront("networkContent");
       stacktrace = await networkContentFront.getStackTrace(
         actor.stacktraceResourceId
       );
@@ -663,6 +680,22 @@ class IceCatDataProvider {
     return payload.requestHeaders;
   }
 
+  /**
+   * Handles additional information received for a "responseHeaders" packet.
+   *
+   * @param {object} response the message received from the server.
+   */
+  async onEarlyHintsResponseHeaders(response) {
+    const payload = await this.updateRequest(response.from, {
+      earlyHintsResponseHeaders: response,
+    });
+    this.emitForTests(
+      TEST_EVENTS.RECEIVED_EARLY_HINTS_RESPONSE_HEADERS,
+      response
+    );
+    return payload.earlyHintsResponseHeaders;
+  }
+
   /**
    * Handles additional information received for a "responseHeaders" packet.
    *
